module ServiceLayerRest
  module ResponseHelper
    THIS_SOURCE = "Rails"
    NO_DATA = "Keine Daten"
    NO_SOURCE = "?"
    NO_SOURCE_DETAILS = "-"

    def check_response_params(response)
        # Check request/response
        return {} if response.nil?

        # Check status
        return {} if response[Constants::STATUS_KEY].nil? || response[Constants::STATUS_KEY] != Rack::Utils::SYMBOL_TO_STATUS_CODE[:ok]

        # Check data/user
        return {} if response[Constants::DATA_KEY].nil? || !response[Constants::DATA_KEY].is_a?(Hash) || response[Constants::DATA_KEY].empty?

        {
            Constants::USERDATA_KEY => response[Constants::DATA_KEY]
        }
    end

    def check_header_params
        # Check request/response
        return {} if request.nil?

        # Check authtype
        return {} if request.headers[Constants::AUTHTYPE_KEY].nil? || request.headers[Constants::AUTHTYPE_KEY].blank?

        # Check mandant id
        return {} if request.headers[Constants::MANDANTID_KEY].nil? || request.headers[Constants::MANDANTID_KEY].blank?

        {
            Constants::AUTHTYPE_KEY => request.headers[Constants::AUTHTYPE_KEY],
            Constants::MANDANT_ID_KEY => request.headers[Constants::MANDANTID_KEY]
        }
    end

    def extract_data_and_state_from_response_string(response_string)
      data = nil
      state = Rack::Utils.status_code(:internal_server_error)

      begin
        response = Oj.load(response_string)

        state = response[Constants::STATUS_KEY] if response[Constants::STATUS_KEY].present?
        data = response[Constants::DATA_KEY] if response[Constants::DATA_KEY].present? || response[Constants::DATA_KEY].in?([true, false])
      rescue Exception => e
        # Silently ignore errors
      end

      return data, state
    end

    def parse_response_string_for_render(response_string, more_data = nil)
      status = Rack::Utils.status_code(:internal_server_error)
      source = NO_SOURCE
      source_details = NO_SOURCE_DETAILS
      data = NO_DATA

      begin
        response = Oj.load(response_string)

        status = response[Constants::STATUS_KEY] if response[Constants::STATUS_KEY].present?
        source = response[Constants::SOURCE_KEY] if response[Constants::SOURCE_KEY].present?
        source_details = response[Constants::SOURCE_DETAILS_KEY] if response[Constants::SOURCE_DETAILS_KEY].present?
        data = response[Constants::DATA_KEY] if response[Constants::DATA_KEY].present? || response[Constants::DATA_KEY].in?([true, false])

        data.merge!(more_data) if more_data.present? && more_data.is_a?(Hash) && data.is_a?(Hash)
      rescue Exception => e
        return parse_exception_for_render e
      end

      create_json_response(status, source, source_details, data)
    end

    def parse_exception_for_render(e)
      print_backtrace e if (Rails.env == "development")
      create_json_error_response(e.message)
    end

    def bad_params_for_render(param_key)
      create_json_response(:bad_request, THIS_SOURCE, "EUVIS.Units.Exceptions.EBadParams", "Parameter für '#{param_key}' sind fehlerhaft.")
    end

    def empty_list_for_render(name_of_empty_list)
      create_json_response(:not_found, THIS_SOURCE, "EUVIS.Units.Exceptions.EListIsEmpty", "Die Liste der #{name_of_empty_list} ist leer.")
    end

    def create_json_error_response(message)
      create_json_response(:internal_server_error, THIS_SOURCE, "EUVIS.Units.Exceptions.EStatusCodeException", (message.nil? || !message.instance_of?(String)) ? "Unbekannter Fehler." : message)
    end

    def create_json_response(status, source, source_details, data)
      puts "Response => :#{Constants::STATUS_KEY} => #{status} / :#{Constants::SOURCE_KEY} => #{source} / :#{Constants::SOURCE_DETAILS_KEY} => #{source_details} / :#{Constants::DATA_KEY} => #{(data.class == Hash || data.class == Array) ? "..." : data}" if $debug_output

      {
        :json => {
          Constants::SOURCE_KEY.to_sym => source,
          Constants::SOURCE_DETAILS_KEY.to_sym => source_details,
          Constants::STATUS_KEY.to_sym => Rack::Utils.status_code(status),
          Constants::DATA_KEY.to_sym => data,
        },
        Constants::STATUS_KEY.to_sym => status,
      }
    end

    def check_params(string_array, int_array)
      return "" if string_array.empty? && int_array.empty?

      string_array.each_with_object(params) do |key, obj|
        return key if obj[key].blank?
      end

      int_array.each_with_object(params) do |key, obj|
        return key if obj[key] == nil 
        value = obj[key]
        value = value.to_i if value.is_a?(String) && value != '' && value.match(/^-?\d+$/)  
        return key if !value.is_a?(Numeric)
      end

      ""
    end

    def check_params_for_tlist(params, main_key, name_of_empty_list, allow_empty = false)
      [main_key].each_with_object(params) do |key, obj|
        return empty_list_for_render(name_of_empty_list) if obj[key][:items].nil? || (obj[key][:items].length == 0 && !allow_empty)

        return bad_params_for_render(key) if obj[key].blank?
        obj[key][:items].each do |item|
          item.each do |inner_key, inner_obj|
            next if inner_obj.in?([true, false])
            return bad_params_for_render("#{key} - #{inner_key}") if inner_obj.nil?
          end
        end
        return bad_params_for_render("#{key} - sortType") if obj[key][:sortType].nil?
        return bad_params_for_render("#{key} - listHelper") if (obj[key][:listHelper].nil? || obj[key][:listHelper].blank?)

        return empty_list_for_render(name_of_empty_list) if (obj[key][:listHelper][0].to_i < 1 && !allow_empty)
        
        return bad_params_for_render("#{key} - sortComparer") if obj[key][:sortComparer].nil?
      end

      true
    end

    def print_backtrace(e)
        puts "Error during processing: #{$!}"
        puts "Backtrace:\n\t#{e.backtrace.join("\n\t")}"
    end

    def start_timer
      $performance_output == true ? Time.now : nil
    end

    def stop_timer(time, text)
      puts("#{text} => #{Time.now - time}") if $performance_output && time != nil
    end
  end
end
